@@ -42,7 +42,7 @@ namespace Breeze
 
     //____________________________________________________________________
     Helper::Helper( KSharedConfig::Ptr config ):
-        _config( config )
+        _config( std::move( config ) )
     { init(); }
 
     //____________________________________________________________________
@@ -128,7 +128,7 @@ namespace Breeze
     {
 
         QColor outline( palette.color( QPalette::Inactive, QPalette::Highlight ) );
-        QColor focus( palette.color( QPalette::Active, QPalette::Highlight ) );
+        const QColor &focus = palette.color( QPalette::Active, QPalette::Highlight );
 
         if( mode == AnimationFocus )
         {
@@ -436,7 +436,7 @@ namespace Breeze
         painter->setRenderHints( QPainter::Antialiasing );
         painter->setBrush( Qt::NoBrush );
         painter->setPen( Qt::red );
-        painter->drawRect( QRectF( rect ).adjusted( 0.5, 0.5, -0.5, -0.5 ) );
+        painter->drawRect( strokedRect( rect ) );
         painter->restore();
     }
 
@@ -459,10 +459,9 @@ namespace Breeze
 
             painter->setClipRect( rect );
 
-            QRectF copy( rect );
-            copy.adjust( 0.5, 0.5, -0.5, -0.5 );
+            QRectF copy( strokedRect( rect ) );
 
-            const qreal radius( frameRadius( -1.0 ) );
+            const qreal radius( frameRadius( PenWidth::Frame, -1 ) );
             if( !(sides&SideTop) ) copy.adjust( 0, -radius, 0, 0 );
             if( !(sides&SideBottom) ) copy.adjust( 0, 0, 0, radius );
             if( !(sides&SideLeft) ) copy.adjust( -radius, 0, 0, 0 );
@@ -501,15 +500,15 @@ namespace Breeze
         painter->setRenderHint( QPainter::Antialiasing );
 
         QRectF frameRect( rect.adjusted( 1, 1, -1, -1 ) );
-        qreal radius( frameRadius() );
+        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
         // set pen
         if( outline.isValid() )
         {
 
             painter->setPen( outline );
-            frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius = qMax( radius - 1, qreal( 0.0 ) );
+            frameRect = strokedRect( frameRect );
+            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
 
         } else {
 
@@ -534,8 +533,7 @@ namespace Breeze
         if( !outline.isValid() ) return;
 
         // adjust rect
-        QRectF frameRect( rect );
-        frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
+        QRectF frameRect( strokedRect( rect ) );
 
         // setup painter
         painter->setRenderHint( QPainter::Antialiasing );
@@ -563,7 +561,7 @@ namespace Breeze
 
             case AllSides:
             {
-                const qreal radius( frameRadius( -1.0 ) );
+                const qreal radius( frameRadius( PenWidth::Frame, -1 ) );
                 painter->drawRoundedRect( frameRect, radius, radius );
                 break;
             }
@@ -588,15 +586,15 @@ namespace Breeze
 
             painter->setRenderHint( QPainter::Antialiasing );
             QRectF frameRect( rect );
-            qreal radius( frameRadius() );
+            qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
             // set pen
             if( outline.isValid() )
             {
 
                 painter->setPen( outline );
-                frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-                radius = qMax( radius - 1, qreal( 0.0 ) );
+                frameRect = strokedRect( frameRect );
+                radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
 
             } else painter->setPen( Qt::NoPen );
 
@@ -634,19 +632,16 @@ namespace Breeze
         // copy rect
         QRectF frameRect( rect );
         frameRect.adjust( 1, 1, -1, -1 );
-        qreal radius( frameRadius() );
+        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
         // shadow
         if( sunken ) {
 
             frameRect.translate( 1, 1 );
 
-        } else if( shadow.isValid() ) {
+        } else {
 
-            const qreal shadowRadius = qMax( radius - 1, qreal( 0.0 ) );
-            painter->setPen( QPen( shadow, 2 ) );
-            painter->setBrush( Qt::NoBrush );
-            painter->drawRoundedRect( shadowRect( frameRect ), shadowRadius, shadowRadius );
+            renderRoundedRectShadow( painter, frameRect, shadow, radius );
 
         }
 
@@ -658,8 +653,8 @@ namespace Breeze
             gradient.setColorAt( 1, outline.darker( hasFocus ? 110:103 ) );
             painter->setPen( QPen( QBrush( gradient ), 1.0 ) );
 
-            frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius = qMax( radius - 1, qreal( 0.0 ) );
+            frameRect = strokedRect( frameRect );
+            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -691,26 +686,25 @@ namespace Breeze
         // setup painter
         painter->setRenderHints( QPainter::Antialiasing );
 
-        const QRectF baseRect( rect );
+        const QRectF baseRect( rect.adjusted( 1, 1, -1, -1 ) );
 
         if( sunken )
         {
 
-            const qreal radius( frameRadius() );
+            const qreal radius( frameRadius( PenWidth::NoPen ) );
 
             painter->setPen( Qt::NoPen );
             painter->setBrush( color );
 
-            const QRectF contentRect( baseRect.adjusted( 1, 1, -1, -1 ) );
-            painter->drawRoundedRect( contentRect, radius, radius );
+            painter->drawRoundedRect( baseRect, radius, radius );
 
         } else {
 
-            const qreal radius( frameRadius(-0.5) );
+            const qreal radius( frameRadius( PenWidth::Frame ) );
 
             painter->setPen( color );
             painter->setBrush( Qt::NoBrush );
-            const QRectF outlineRect( baseRect.adjusted( 1.5, 1.5, -1.5, -1.5 ) );
+            const QRectF outlineRect( strokedRect( baseRect ) );
             painter->drawRoundedRect( outlineRect, radius, radius );
 
         }
@@ -726,7 +720,7 @@ namespace Breeze
         if( !outline.isValid() ) return;
 
         // round radius
-        const qreal radius( frameRadius() );
+        const qreal radius( frameRadius( PenWidth::Frame ) );
         const QSizeF cornerSize( 2*radius, 2*radius );
 
         // if rect - tabwidth is even, need to increase tabWidth by 1 unit
@@ -734,8 +728,7 @@ namespace Breeze
         if( !((rect.width() - tabWidth)%2) ) ++tabWidth;
 
         // adjust rect for antialiasing
-        QRectF baseRect( rect );
-        baseRect.adjust( 0.5, 0.5, -0.5, -0.5 );
+        QRectF baseRect( strokedRect( rect ) );
 
         // create path
         QPainterPath path;
@@ -768,15 +761,15 @@ namespace Breeze
         painter->setRenderHint( QPainter::Antialiasing );
 
         QRectF frameRect( rect.adjusted( 1, 1, -1, -1 ) );
-        qreal radius( frameRadius() );
+        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
         // set pen
         if( outline.isValid() )
         {
 
             painter->setPen( outline );
-            frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius = qMax( radius-1, qreal( 0.0 ) );
+            frameRect = strokedRect( frameRect );
+            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -865,7 +858,7 @@ namespace Breeze
         // copy rect and radius
         QRectF frameRect( rect );
         frameRect.adjust( 2, 2, -2, -2 );
-        qreal radius( frameRadius() );
+        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
         // shadow
         if( sunken )
@@ -875,22 +868,18 @@ namespace Breeze
 
         } else {
 
-            painter->setPen( QPen( shadow, 1 ) );
-            painter->setBrush( Qt::NoBrush );
-
-            const qreal shadowRadius( radius + 0.5 );
-            painter->drawRoundedRect( shadowRect( frameRect ).adjusted( -0.5, -0.5, 0.5, 0.5 ), shadowRadius, shadowRadius );
+            renderRoundedRectShadow( painter, frameRect, shadow, radius );
 
         }
 
         // content
         {
 
-            painter->setPen( QPen( color, 1 ) );
+            painter->setPen( QPen( color, PenWidth::Frame ) );
             painter->setBrush( Qt::NoBrush );
 
-            radius = qMax( radius-1, qreal( 0.0 ) );
-            const QRectF contentRect( frameRect.adjusted( 0.5, 0.5, -0.5, -0.5 ) );
+            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
+            const QRectF contentRect( strokedRect( frameRect ) );
             painter->drawRoundedRect( contentRect, radius, radius );
 
         }
@@ -982,19 +971,17 @@ namespace Breeze
 
         } else {
 
-            painter->setPen( QPen( shadow, 1 ) );
-            painter->setBrush( Qt::NoBrush );
-            painter->drawEllipse( shadowRect( frameRect ).adjusted( -0.5, -0.5, 0.5, 0.5 ) );
+            renderEllipseShadow( painter, frameRect, shadow );
 
         }
 
         // content
         {
 
-            painter->setPen( QPen( color, 1 ) );
+            painter->setPen( QPen( color, PenWidth::Frame ) );
             painter->setBrush( Qt::NoBrush );
 
-            const QRectF contentRect( frameRect.adjusted( 0.5, 0.5, -0.5, -0.5 ) );
+            const QRectF contentRect( strokedRect( frameRect ) );
             painter->drawEllipse( contentRect );
 
         }
@@ -1141,12 +1128,10 @@ namespace Breeze
         frameRect.adjust( 1, 1, -1, -1 );
 
         // shadow
-        if( shadow.isValid() && !sunken )
+        if( !sunken )
         {
 
-            painter->setPen( QPen( shadow, 2 ) );
-            painter->setBrush( Qt::NoBrush );
-            painter->drawEllipse( shadowRect( frameRect ) );
+            renderEllipseShadow( painter, frameRect, shadow );
 
         }
 
@@ -1155,7 +1140,7 @@ namespace Breeze
         {
 
             painter->setPen( outline );
-            frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
+            frameRect = strokedRect( frameRect );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -1270,6 +1255,23 @@ namespace Breeze
         
     }
 
+
+    //______________________________________________________________________________
+    void Helper::renderScrollBarBorder(
+        QPainter* painter, const QRect& rect,
+        const QColor& color ) const
+    {
+
+        // content
+        if( color.isValid() )
+        {
+            painter->setPen( Qt::NoPen );
+            painter->setBrush( color );
+            painter->drawRect( rect );
+        }
+
+    }
+
     //______________________________________________________________________________
     void Helper::renderTabBarTab( QPainter* painter, const QRect& rect, const QColor& color, const QColor& outline, Corners corners ) const
     {
@@ -1278,15 +1280,15 @@ namespace Breeze
         painter->setRenderHint( QPainter::Antialiasing, true );
 
         QRectF frameRect( rect );
-        qreal radius( frameRadius() );
+        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
         // pen
         if( outline.isValid() )
         {
 
             painter->setPen( outline );
-            frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius = qMax( radius-1, qreal( 0.0 ) );
+            frameRect = strokedRect( frameRect );
+            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -1308,10 +1310,15 @@ namespace Breeze
         QPolygonF arrow;
         switch( orientation )
         {
-            case ArrowUp: arrow = QVector<QPointF>{QPointF( -4, 2 ), QPointF( 0, -2 ), QPointF( 4, 2 )}; break;
-            case ArrowDown: arrow = QVector<QPointF>{QPointF( -4, -2 ), QPointF( 0, 2 ), QPointF( 4, -2 )}; break;
-            case ArrowLeft: arrow = QVector<QPointF>{QPointF( 2, -4 ), QPointF( -2, 0 ), QPointF( 2, 4 )}; break;
-            case ArrowRight: arrow = QVector<QPointF>{QPointF( -2, -4 ), QPointF( 2, 0 ), QPointF( -2, 4 )}; break;
+            /* The inner points of the normal arrows are not on half pixels because
+             * they need to have an even width (up/down) or height (left/right).
+             * An even width/height makes them easier to align with other UI elements.
+             */
+            case ArrowUp: arrow = QVector<QPointF>{QPointF( -4.5, 1.5 ), QPointF( 0, -3 ), QPointF( 4.5, 1.5 )}; break;
+            case ArrowDown: arrow = QVector<QPointF>{QPointF( -4.5, -1.5 ), QPointF( 0, 3 ), QPointF( 4.5, -1.5 )}; break;
+            case ArrowLeft: arrow = QVector<QPointF>{QPointF( 1.5, -4.5 ), QPointF( -3, 0 ), QPointF( 1.5, 4.5 )}; break;
+            case ArrowRight: arrow = QVector<QPointF>{QPointF( -1.5, -4.5 ), QPointF( 3, 0 ), QPointF( -1.5, 4.5 )}; break;
+            case ArrowDown_Small: arrow = QVector<QPointF>{QPointF( 1.5, 3.5 ), QPointF( 3.5, 5.5 ), QPointF( 5.5, 3.5 )}; break;
             default: break;
         }
 
@@ -1319,7 +1326,10 @@ namespace Breeze
         painter->setRenderHints( QPainter::Antialiasing );
         painter->translate( QRectF( rect ).center() );
         painter->setBrush( Qt::NoBrush );
-        painter->setPen( QPen( color, 1.1 ) );
+        QPen pen( color, PenWidth::Symbol );
+        pen.setCapStyle(Qt::SquareCap);
+        pen.setJoinStyle(Qt::MiterJoin);
+        painter->setPen( pen );
         painter->drawPolyline( arrow );
         painter->restore();
    }
@@ -1359,7 +1369,7 @@ namespace Breeze
 
         pen.setCapStyle( Qt::RoundCap );
         pen.setJoinStyle( Qt::MiterJoin );
-        pen.setWidthF( 1.1*qMax(1.0, 18.0/rect.width() ) );
+        pen.setWidthF( PenWidth::Symbol*qMax(1.0, 18.0/rect.width() ) );
         painter->setPen( pen );
 
         switch( buttonType )
@@ -1408,7 +1418,64 @@ namespace Breeze
         painter->restore();
 
     }
+    
+    //______________________________________________________________________________
+    void Helper::renderRoundedRectShadow( QPainter* painter, const QRectF& rect, const QColor& color, qreal radius ) const
+    {
+        if( !color.isValid() ) return;
+        
+        painter->save();
+        
+        qreal translation = 0.5 * PenWidth::Shadow; // Translate for the pen
+        
+        /* Clipping prevents shadows from being visible inside checkboxes.
+         * Clipping away unneeded parts here also improves performance by 40-60%
+         * versus using just an outline of a rectangle.
+         * Tested by looking at the paint analyser in GammaRay.
+         */
+        // Right side
+        QRegion clip( rect.right() - std::ceil( radius ), rect.top(), 
+                      std::ceil( radius ) + PenWidth::Shadow, rect.height() );
+        // Bottom side
+        clip = clip.united( QRegion( rect.left(), rect.bottom() - std::ceil( radius ), 
+                                     rect.width(), std::ceil( radius ) + PenWidth::Shadow ) );
 
+        painter->setClipRegion( clip );
+        painter->setPen( color );
+        painter->setBrush( Qt::NoBrush );
+        painter->drawRoundedRect( rect.translated( translation, translation ), radius, radius );
+        
+        painter->restore();
+    }
+    
+    //______________________________________________________________________________
+    void Helper::renderEllipseShadow( QPainter* painter, const QRectF& rect, const QColor& color ) const
+    {
+        if( !color.isValid() ) return;
+        
+        painter->save();
+
+        // Clipping does not improve performance here
+
+        qreal adjustment = 0.5 * PenWidth::Shadow; // Adjust for the pen
+
+        qreal radius = rect.width() / 2 - adjustment;
+        
+        /* The right side is offset by +0.5 for the visible part of the shadow.
+         * The other sides are offset by +0.5 or -0.5 because of the pen.
+         */
+        QRectF shadowRect = rect.adjusted( adjustment, adjustment, adjustment, -adjustment );
+        
+        painter->translate( rect.center() );
+        painter->rotate( 45 );
+        painter->translate( -rect.center() );
+        painter->setPen( color );
+        painter->setBrush( Qt::NoBrush );
+        painter->drawRoundedRect( shadowRect, radius, radius );
+        
+        painter->restore();
+    }
+    
     //______________________________________________________________________________
     bool Helper::isX11()
     {
@@ -1437,8 +1504,22 @@ namespace Breeze
     }
 
     //______________________________________________________________________________
-    QRectF Helper::shadowRect( const QRectF& rect ) const
-    { return rect.adjusted( 0.5, 0.5, -0.5, -0.5 ).translated( 0.5, 0.5 ); }
+    QRectF Helper::strokedRect( const QRectF &rect, const int penWidth ) const
+    {
+        /* With a pen stroke width of 1, the rectangle should have each of its
+         * sides moved inwards by half a pixel. This allows the stroke to be
+         * pixel perfect instead of blurry from sitting between pixels and
+         * prevents the rectangle with a stroke from becoming larger than the
+         * original size of the rectangle.
+         */
+        qreal adjustment = 0.5 * penWidth;
+        return QRectF( rect ).adjusted( adjustment, adjustment, -adjustment, -adjustment );
+    }
+    
+    QRectF Helper::strokedRect( const QRect &rect, const int penWidth ) const
+    {
+        return strokedRect(QRectF(rect), penWidth);
+    }
 
     //______________________________________________________________________________
     QPainterPath Helper::roundedPath( const QRectF& rect, Corners corners, qreal radius ) const
